# -*- coding: utf-8 -*-


from pathlib import Path
from unittest.mock import patch

import pytest

from clifs.__main__ import main
from tests.common.utils_testing import (
    assert_files_present,
    parametrize_default_ids,
    substr_in_dir_names,
)


def create_test_cfg(path_cfg, path_output):
    path_cfg = Path(path_cfg)
    path_output = Path(path_output)

    with path_cfg.open("r") as file:
        text_cfg = file.read()

    text_cfg = text_cfg.replace("WHEREAMI", str(path_output))

    path_output = path_output / path_cfg.name
    with path_output.open("w") as file:
        file.write(text_cfg)
    return path_output


@pytest.fixture(scope="function")
def cfg_testrun(dir_testrun):
    path_cfg_template = Path(__file__).parent / "cfg_template.csv"
    path_cfg_test = create_test_cfg(path_cfg_template, dir_testrun)
    return path_cfg_test


@parametrize_default_ids("from_cfg", [False, True])
@parametrize_default_ids("delete", [False, True])
@parametrize_default_ids("dry_run", [False, True])
def test_backup(
    cfg_testrun,
    dirs_source,
    dirs_dest,
    dirs_source_ref,
    dirs_dest_ref,
    from_cfg,
    delete,
    dry_run,
):
    # run the actual function to test
    if from_cfg:
        patch_args = ["clifs", "backup", "--cfg_file", str(cfg_testrun)]
        if delete:
            patch_args.append("--delete")
        if dry_run:
            patch_args.append("--dry_run")

        with patch("sys.argv", patch_args):
            main()

    else:
        for idx_dir in range(len(dirs_source)):
            patch_args = [
                "clifs",
                "backup",
                "--dir_source",
                str(dirs_source[idx_dir]),
                "--dir_dest",
                str(dirs_dest[idx_dir]),
            ]
            if delete:
                patch_args.append("--delete")
            if dry_run:
                patch_args.append("--dry_run")

            with patch("sys.argv", patch_args):
                main()

    print("Backup went through.")

    if not dry_run:
        # check for proper updating and deleting
        for idx_dir in range(len(dirs_source)):
            assert_files_present(dirs_source[idx_dir], dirs_dest[idx_dir])
            if delete:
                assert not substr_in_dir_names(
                    dirs_dest[idx_dir]
                ), "Files only present in destination dir have not been deleted."
            else:
                assert substr_in_dir_names(
                    dirs_dest[idx_dir]
                ), "Files only present in destination dir have been deleted."
    else:
        # check for dest dir integrity
        for idx_dir in range(len(dirs_source)):
            assert_files_present(dirs_dest[idx_dir], dirs_dest_ref[idx_dir])

    # check for source dir integrity
    for idx_dir in range(len(dirs_source)):
        assert_files_present(dirs_source[idx_dir], dirs_source_ref[idx_dir])



# -*- coding: utf-8 -*-


import re
from argparse import Namespace
from unittest.mock import patch

from clifs.__main__ import main
from clifs.plugins.disc_usage import DiscUsageExplorer
from tests.common.utils_testing import escape_rich_style, parametrize_default_ids


@parametrize_default_ids("usage_info", [(1000, 400, 600), (100000, 70000, 30000)])
def test_disc_usage_numbers(dirs_empty, usage_info):
    # run the actual function to test
    with patch("shutil.disk_usage", return_value=(usage_info)):
        usage_explorer = DiscUsageExplorer(
            Namespace(
                dirs=[str(x) for x in dirs_empty],
            )
        )

    for dir in dirs_empty:
        space_total = usage_explorer._dict_usage[str(dir)].total
        space_used = usage_explorer._dict_usage[str(dir)].used
        space_free = usage_explorer._dict_usage[str(dir)].free

        assert (
            space_total == usage_info[0]
        ), f"Total disc space does not match: exp={usage_info[0]}, act={space_total}"
        assert (
            space_used == usage_info[1]
        ), f"Used disc space does not match: exp={usage_info[1]}, act={space_used}"
        assert (
            space_free == usage_info[2]
        ), f"Total disc space does not match: exp={usage_info[2]}, act={space_free}"


@parametrize_default_ids("usage_info", [(1000, 400, 600), (100000, 70000, 30000)])
def test_disc_usage_output(dirs_empty, usage_info, capfd):
    # run the actual function to test
    with patch("sys.argv", ["clifs", "du"] + [str(x) for x in dirs_empty]), patch(
        "shutil.disk_usage", return_value=(usage_info)
    ):
        main()

    captured = capfd.readouterr()
    for sp_type, sp_val in zip(["total", "used", "free"], usage_info):
        assert len(
            re.findall(
                rf"{sp_type}: *{sp_val / 1024:.2f} KB", escape_rich_style(captured.out)
            )
        ) == len(dirs_empty)
    perc_used = round(100.0 * usage_info[1] / float(usage_info[0]), 1)
    assert len(re.findall(rf"{perc_used:5}%", escape_rich_style(captured.out))) == len(
        dirs_empty
    )



# -*- coding: utf-8 -*-

from unittest.mock import patch

from clifs.__main__ import main
from tests.common.utils_testing import (
    assert_files_present,
    parametrize_default_ids,
    substr_in_dir_names,
)


@parametrize_default_ids(
    ("pattern", "replacement"), [("file", "SUBSTITUTE"), (".txt", "123456")]
)
def test_rename(dirs_source, dirs_source_ref, pattern, replacement):
    # run the actual function to test

    for idx_dir, dir in enumerate(dirs_source):
        patch_args = [
            "clifs",
            "ren",
            str(dir),
            "--pattern",
            pattern,
            "--replacement",
            replacement,
            "--recursive",
        ]
        with patch("sys.argv", patch_args), patch("builtins.input", return_value="yes"):
            main()

        print(f"Renaming of {dir.name} went through.")

        assert not substr_in_dir_names(dir, sub_str=pattern, files_only=True)

        # revert and check conistency
        patch_args = [
            "clifs",
            "ren",
            str(dir),
            "--pattern",
            replacement,
            "--replacement",
            pattern,
            "--recursive",
            "--skip_preview",
        ]
        with patch("sys.argv", patch_args):
            main()

        print(f"Re-renaming of {dir.name} went through.")

        assert not substr_in_dir_names(dir, sub_str=replacement, files_only=True)

        assert_files_present(dir_source=dirs_source_ref[idx_dir], dir_ref=dir)



"""Main entry point calling plugins"""


import argparse
import sys
from importlib.metadata import entry_points
from typing import Dict, Type

import clifs
from clifs import ClifsPlugin
from clifs.utils_cli import CONSOLE, set_style


def main() -> None:
    """Main entry point calling plugins installed as 'clifs.plugins'"""
    CONSOLE.print(
        set_style(f"running `clifs` version: {clifs.__version__}", "bright_black")
    )

    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    commands = parser.add_subparsers(title="Available plugins", dest="plugin")

    plugins: Dict[str, Type[ClifsPlugin]] = {}
    for entry_point in entry_points()["clifs.plugins"]:
        plugins[entry_point.name] = entry_point.load()
        subparser = commands.add_parser(
            entry_point.name, help=plugins[entry_point.name].__doc__
        )
        plugins[entry_point.name].init_parser(parser=subparser)

    if len(sys.argv) == 1:
        print("No function specified. Have a look at the awesome options:")
        parser.print_help()
        sys.exit(1)

    args = parser.parse_args()
    plugin = plugins[args.plugin](args)
    plugin.run()


if __name__ == "__main__":
    main()



"""Clifs plugin for regex-based file renaming"""

import re
import sys
from argparse import ArgumentParser, Namespace
from collections import Counter
from pathlib import Path
from typing import List, Set

from clifs import ClifsPlugin
from clifs.utils_cli import cli_bar, print_line, set_style, user_query
from clifs.utils_fs import INDENT, FileGetterMixin, get_unique_path


class FileRenamer(ClifsPlugin, FileGetterMixin):
    """
    Regex-based file renaming.
    """

    pattern: str
    replacement: str
    skip_preview: bool

    @classmethod
    def init_parser(cls, parser: ArgumentParser) -> None:
        """
        Adding arguments to an argparse parser. Needed for all clifs_plugins.
        """
        # add args from FileGetterMixin to arg parser
        super().init_parser_mixin(parser)

        parser.add_argument(
            "-pt",
            "--pattern",
            default=".*",
            help="Pattern identifying the substring to be replaced. "
            "Supports syntax for `re.sub` from regex module "
            "(https://docs.python.org/3/library/re.html).",
        )
        parser.add_argument(
            "-rp",
            "--replacement",
            default="",
            help="String to use as replacement. "
            "You can use \\1 \\2 etc. to refer to matching groups. "
            "E.g. a pattern like '(.+)\\.(.+)' in combination "
            "with a replacement like '\\1_suffix.\\2' will append suffixes. "
            "Defaults to empty string.",
        )
        parser.add_argument(
            "-sp",
            "--skip_preview",
            action="store_true",
            help="Skip preview on what would happen and rename right away. "
            "Only for the brave...",
        )

    def __init__(self, args: Namespace) -> None:
        super().__init__(args)
        self.files2process: List[Path] = self.get_files()
        self.counter = Counter(files2process=len(self.files2process))

    def run(self) -> None:
        self.exit_if_nothing_to_process(self.files2process)

        if not self.skip_preview:
            self.rename_files(preview_mode=True)
            if not user_query(
                'If you want to apply renaming, give me a "yes" or "y" now!'
            ):
                self.console.print("Will not rename for now. See you soon.")
                sys.exit(0)

        self.rename_files(preview_mode=False)

    def rename_files(
        self,
        preview_mode: bool = True,
    ) -> None:
        self.console.print(f"Renaming {self.counter['files2process']} files.")
        print_line(self.console)
        files_to_be_added: Set[Path] = set()
        files_to_be_deleted: Set[Path] = set()
        if preview_mode:
            self.console.print("Preview:")

        num_file = 0
        for num_file, path_file in enumerate(self.files2process, 1):
            name_old = path_file.name
            name_new = re.sub(self.pattern, self.replacement, name_old)
            message_rename = f"{name_old:35} -> {name_new:35}"

            # skip files if renaming would result in bad characters
            found_bad_chars = self.find_bad_char(name_new)
            if found_bad_chars:
                message_rename += set_style(
                    f"{INDENT}Error: not doing renaming as it would result "
                    f"in bad characters: '{','.join(found_bad_chars)}'",
                    "error",
                )
                self.counter["bad_results"] += 1
                self.print_rename_message(
                    message_rename,
                    num_file,
                    preview_mode=preview_mode,
                )
                continue

            # make sure resulting paths are unique
            path_file_new = path_file.parent / name_new
            path_file_unique = get_unique_path(
                path_file_new,
                set_taken=files_to_be_added,
                set_free=files_to_be_deleted | {path_file},
            )

            if path_file_new != path_file_unique:
                path_file_new = path_file_unique
                name_new = path_file_unique.name
                message_rename = f"{name_old:35} -> {name_new:35}"
                message_rename += set_style(
                    f"{INDENT}Warning: name result would already exist. "
                    "Adding number suffix.",
                    "warning",
                )
                self.counter["name_conflicts"] += 1

            # skip files that are not renamed
            if path_file_new == path_file:
                message_rename = set_style(message_rename, "bright_black")
                self.print_rename_message(
                    message_rename,
                    num_file,
                    preview_mode=preview_mode,
                )
                continue

            self.print_rename_message(
                message_rename,
                num_file,
                preview_mode=preview_mode,
            )
            if not preview_mode:
                path_file.rename(path_file_new)
                self.counter["files_renamed"] += 1
            else:
                files_to_be_added.add(path_file_new)
                if path_file_new in files_to_be_deleted:
                    files_to_be_deleted.remove(path_file_new)
                files_to_be_deleted.add(path_file)

        if self.counter["bad_results"] > 0:
            self.console.print(
                set_style(
                    f"Warning: {self.counter['bad_results']} out of "
                    f"{self.counter['files2process']} files not renamed as it would "
                    "result in bad characters.",
                    "warning",
                )
            )

        if self.counter["name_conflicts"] > 0:
            self.console.print(
                set_style(
                    f"Warning: {self.counter['name_conflicts']} out of "
                    f"{self.counter['files2process']} renamings would have resulted in "
                    "name conflicts. Added numbering suffices to get unique names.",
                    "warning",
                )
            )

        if not preview_mode:
            self.console.print(
                f"Hurray, {num_file} files have been processed, "
                f"{self.counter['files_renamed']} have been renamed."
            )
        print_line(self.console)

    def print_rename_message(
        self,
        message: str,
        num_file: int,
        preview_mode: bool = False,
        space_prefix: str = "    ",
    ) -> None:
        if preview_mode:
            self.console.print(space_prefix + message)
        else:
            cli_bar(
                num_file,
                self.counter["files2process"],
                suffix=space_prefix + message,
                console=self.console,
            )

    @staticmethod
    def find_bad_char(string: str) -> List[str]:
        """Check stings for characters causing problems in windows file system."""
        bad_chars = r"~“#%&*:<>?/\{|}"
        return [x for x in bad_chars if x in string]



"""Clifs plugin to show directory trees"""

from abc import ABC, abstractmethod
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Any, List, Optional

from rich.console import Console

from clifs import ClifsPlugin
from clifs.utils_cli import set_style, size2str

PIPE = "│"
ELBOW = "└──"
TEE = "├──"
PIPE_PREFIX = "│   "
SPACE_PREFIX = "    "
SPACE_SIZE = " "


class Entry(ABC):
    """
    Base class for entries in a DirectoryTree
    """

    def __init__(  # pylint: disable=too-many-arguments
        self,
        path: Path,
        prefix: str = "",
        connector: str = "",
        depth: int = 0,
        plot_size: bool = True,
    ):
        self.path = path
        self.prefix = prefix
        self.connector = connector
        self.depth = depth
        self.plot_size = plot_size

        self.name: str = self.path.name
        self.size: Optional[float] = None

    @abstractmethod
    def __str__(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_size(self) -> Optional[float]:
        """Get the overall size of the entry"""
        raise NotImplementedError


class File(Entry):
    """Representing files in a Directory Tree"""

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self.size: Optional[float] = self.get_size() if self.plot_size else None

    def get_size(self) -> Optional[float]:
        try:
            return self.path.stat().st_size
        except FileNotFoundError:
            file_long = Path(
                "\\\\?\\" + str(self.path)
            )  # handle long paths in windows systems
            return file_long.stat().st_size

    def __str__(self) -> str:
        string = f"{self.prefix}{self.connector} {self.name}"
        if self.plot_size and self.size is not None:
            string += SPACE_SIZE + size2str(self.size)
        return string


class Folder(Entry):
    """
    Representing folders in a DirectoryTree.
    """

    def __init__(
        self,
        dirs_only: bool = False,
        depth_th: Optional[int] = None,
        console: Console = Console(),
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        self.dirs_only = dirs_only
        self.depth_th = depth_th
        self.console = console

        self.have_access: bool = True
        self.children: List[Entry] = []

        if (self.depth_th is None or self.depth < self.depth_th) or self.plot_size:
            self.children = self.get_children()

        self.size: Optional[float] = self.get_size() if self.plot_size else None

    def get_children(self) -> List[Entry]:
        children: List[Entry] = []

        items = list(self.path.iterdir())
        try:
            items = sorted(items, key=lambda item: (not item.is_file(), str(item)))
            for num_item, item in enumerate(items, 1):
                child_connector = TEE if num_item < len(items) else ELBOW

                child_prefix = self.prefix
                if self.connector == TEE:  # not last dir
                    child_prefix += PIPE_PREFIX
                elif self.connector == ELBOW:  # last dir
                    child_prefix += SPACE_PREFIX

                if item.is_file():
                    if not self.dirs_only or self.plot_size:
                        children.append(
                            File(
                                path=item,
                                prefix=child_prefix,
                                connector=child_connector,
                                depth=self.depth + 1,
                                plot_size=self.plot_size,
                            )
                        )
                if item.is_dir():
                    children.append(
                        Folder(
                            path=item,
                            prefix=child_prefix,
                            connector=child_connector,
                            depth=self.depth + 1,
                            depth_th=self.depth_th,
                            dirs_only=self.dirs_only,
                            plot_size=self.plot_size,
                        )
                    )
            return children

        except PermissionError as err:
            self.console.print(
                set_style(
                    f'Error: no permission to access "{self.path}". '
                    "Size calculations of parent directories could be off.",
                    "error",
                )
            )
            self.console.print(set_style(f'Error message: "{err}"', "error"))
            self.have_access = False
            return []

    def get_size(self) -> Optional[float]:
        if not self.have_access:
            return None
        size = 0.0
        for child in self.children:
            size += child.size if child.size is not None else size
        return size

    def __str__(self) -> str:
        string = f"{self.prefix}{self.connector} " if self.depth != 0 else ""
        string += set_style(self.name, "folder")

        if self.plot_size and self.size is not None:
            string += SPACE_SIZE + size2str(self.size)
        elif not self.have_access:
            string += SPACE_SIZE + set_style("no access", "error")

        if (self.depth_th is None or self.depth < self.depth_th) and self.children:
            if self.dirs_only:
                dir_children = [
                    child.__str__()
                    for child in self.children
                    if isinstance(child, Folder)
                ]
                if dir_children:
                    string += "\n" + "\n".join(dir_children)
            else:
                string += "\n" + "\n".join([child.__str__() for child in self.children])
        return string


class DirectoryTree(ClifsPlugin):
    """
    Display a tree of the file system including item sizes.
    """

    root_dir: Path
    dirs_only: bool
    hide_sizes: bool
    depth: int

    @staticmethod
    def init_parser(parser: ArgumentParser) -> None:
        """
        Adding arguments to an argparse parser. Needed for all clifs_plugins.
        """
        parser.add_argument(
            "root_dir",
            type=Path,
            default=".",
            nargs="?",
            help="Root directory to generate tree",
        )
        parser.add_argument(
            "-do",
            "--dirs_only",
            action="store_true",
            default=False,
            help="Get info only on directories",
        )
        parser.add_argument(
            "-hs",
            "--hide_sizes",
            action="store_true",
            default=False,
            help="Hide size information",
        )
        parser.add_argument(
            "-d",
            "--depth",
            type=int,
            default=None,
            help="Max depth to which the tree is plotted",
        )

    def __init__(self, args: Namespace) -> None:
        super().__init__(args)
        self.dir: Folder = Folder(
            path=self.root_dir.resolve(),
            plot_size=not self.hide_sizes,
            depth_th=self.depth,
            dirs_only=self.dirs_only,
            console=self.console,
        )

    def __str__(self) -> str:
        return self.dir.__str__()

    def __rich__(self) -> str:
        return self.dir.__str__()

    def run(self) -> None:
        self.console.print(self, highlight=False)



"""Definition of the ClifsPlugin interface"""


from abc import ABC, abstractmethod
from argparse import ArgumentParser, Namespace

from clifs.utils_cli import CONSOLE


class ClifsPlugin(ABC):
    # Using ABC instead of typing.Protocol here as Protocols break resolving of
    # the __init__ method, see:
    # https://bugs.python.org/issue44807
    """
    Class to inherit for clifs plugins.
    """

    @staticmethod
    @abstractmethod
    def init_parser(parser: ArgumentParser) -> None:
        """
        Adding arguments to an argparse parser. Needed for all clifs plugins.
        """
        raise NotImplementedError

    def __init__(self, args: Namespace) -> None:
        """
        Converts arguments to instance attributes.
        """
        for arg in vars(args):
            setattr(self, arg, getattr(args, arg))
        self.console = CONSOLE

    @abstractmethod
    def run(self) -> None:
        """
        Running the plugin. Needed for all clifs plugins.
        """
        raise NotImplementedError



"""Clifs plugin for file deletion"""

import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import List

from clifs import ClifsPlugin
from clifs.utils_cli import CONSOLE, cli_bar, print_line, user_query
from clifs.utils_fs import FileGetterMixin


class FileDeleter(ClifsPlugin, FileGetterMixin):
    """
    Delete files
    """

    skip_preview: bool

    @classmethod
    def init_parser(cls, parser: ArgumentParser) -> None:
        """
        Adding arguments to an argparse parser. Needed for all clifs_plugins.
        """
        # add args from FileGetterMixin to arg parser
        super().init_parser_mixin(parser)

        parser.add_argument(
            "-sp",
            "--skip_preview",
            action="store_true",
            help="Skip preview on what would happen and delete right away. "
            "For the brave only...",
        )

    def __init__(self, args: Namespace) -> None:
        super().__init__(args)
        self.console = CONSOLE
        self.files2process: List[Path] = self.get_files()

    def run(self) -> None:
        self.exit_if_nothing_to_process(self.files2process)

        if not self.skip_preview:
            self.console.print("Preview:")
            self.delete_files(dry_run=True)
            if not user_query(
                'If you want to delete for real, give me a "yes" or "y" now!'
            ):
                print("Will not delete for now. See you soon.")
                sys.exit(0)
        self.delete_files(dry_run=False)

    def delete_files(self, dry_run: bool = False) -> None:
        num_files2process = len(self.files2process)
        print_line(self.console)
        if dry_run:
            self.console.print(f"Would delete the following {num_files2process} files:")
        else:
            self.console.print(f"Deleting {num_files2process} files:")

        num_file = 0
        for num_file, path_file in enumerate(self.files2process, 1):
            if dry_run:
                self.console.print(f"    {path_file.name}")
            else:
                path_file.unlink(missing_ok=True)
                cli_bar(
                    num_file,
                    num_files2process,
                    suffix=f"deleted. Last: {path_file.name}",
                )
        print_line(self.console)
        if not dry_run:
            print(f"Hurray, {num_file} files have been deleted.")



# -*- coding: utf-8 -*-


from clifs.utils_fs import FileGetterMixin
from tests.common.utils_testing import parametrize_default_ids


@parametrize_default_ids("filter_str", [".txt", "2", ""])
@parametrize_default_ids("recursive", [True, False])
@parametrize_default_ids(
    ["path_filterlist", "header_filterlist", "sep_filterlist"],
    [
        (None, None, None),
        ("path_filterlist_txt", None, None),
        ("path_filterlist_csv", "filter", ","),
        ("path_filterlist_tsv", "filter", "\t"),
    ],
)
def test_file_getter(
    dirs_source,
    recursive,
    filter_str,
    path_filterlist,
    header_filterlist,
    sep_filterlist,
    request,
):
    for dir in dirs_source:
        # run the actual function to test
        file_getter = FileGetterMixin()

        file_getter.dir_source = dir
        file_getter.recursive = recursive
        file_getter.filterlist = (
            path_filterlist
            if path_filterlist is None
            else request.getfixturevalue(path_filterlist)
        )
        file_getter.filterlistheader = header_filterlist
        file_getter.filterlistsep = sep_filterlist
        file_getter.filterstring = filter_str

        files_found = file_getter.get_files()

        pattern = f"*{filter_str}*" if filter_str else "*"

        if path_filterlist is None:
            if recursive:
                assert files_found == [
                    file for file in dir.rglob(pattern) if not file.is_dir()
                ]
            else:
                assert files_found == [
                    file for file in dir.glob(pattern) if not file.is_dir()
                ]

        else:
            exp_files = ["L1_file_2.txt", "L2_file_1.txt", "L3_file_3.txt"]
            if recursive:
                assert files_found == [
                    file
                    for file in dir.rglob(pattern)
                    if not file.is_dir() and file.name in exp_files
                ]
            else:
                assert files_found == [
                    file
                    for file in dir.glob(pattern)
                    if not file.is_dir() and file.name in exp_files
                ]



# -*- coding: utf-8 -*-


from unittest.mock import patch

from clifs.__main__ import main
from tests.common.utils_testing import assert_files_present, parametrize_default_ids


@parametrize_default_ids(
    ["skip_existing", "keep_all"], [(False, True), (True, False), (False, False)]
)
@parametrize_default_ids("dryrun", [False, True])
@parametrize_default_ids("flatten", [False, True])
def test_copy(
    dirs_source,
    dirs_dest,
    dirs_source_ref,
    dirs_dest_ref,
    dirs_empty,
    dirs_flatten_dir_source_ref,
    dirs_keep_dir_source2dest_ref,
    dryrun,
    skip_existing,
    keep_all,
    flatten,
):
    n_test_dirs = len(dirs_source)
    # run the actual function to test
    for idx_dir in range(n_test_dirs):
        patch_args = [
            "clifs",
            "cp",
            str(dirs_source[idx_dir]),
            str(dirs_empty[idx_dir] if flatten else dirs_dest[idx_dir]),
            "--recursive",
        ]

        if skip_existing:
            patch_args.append("--skip_existing")
        if keep_all:
            patch_args.append("--keep_all")
        if flatten:
            patch_args.append("--flatten")
        if dryrun:
            patch_args.append("--dryrun")

        with patch("sys.argv", patch_args):
            main()
    print("Copy went through.")

    if not dryrun:
        # check for proper updating and deleting
        if flatten:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_empty[idx_dir],
                    dirs_flatten_dir_source_ref[idx_dir],
                    check_mtime=False,
                )
                assert_files_present(
                    dirs_flatten_dir_source_ref[idx_dir],
                    dirs_empty[idx_dir],
                    check_mtime=False,
                )
        elif keep_all:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_keep_dir_source2dest_ref[idx_dir],
                    dirs_dest[idx_dir],
                    check_mtime=False,
                )
                assert_files_present(
                    dirs_dest[idx_dir],
                    dirs_keep_dir_source2dest_ref[idx_dir],
                    check_mtime=False,
                )
        else:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_source[idx_dir],
                    dirs_dest[idx_dir],
                    check_mtime=not skip_existing,
                )
    else:
        # check for dest dir integrity
        for idx_dir in range(n_test_dirs):
            assert_files_present(dirs_dest[idx_dir], dirs_dest_ref[idx_dir])

    # check for source dir integrity
    for idx_dir in range(n_test_dirs):
        assert_files_present(dirs_source[idx_dir], dirs_source_ref[idx_dir])


@parametrize_default_ids(
    "skip_existing, keep_all", [(False, True), (True, False), (False, False)]
)
@parametrize_default_ids("dryrun", [False, True])
@parametrize_default_ids("flatten", [False, True])
def test_move(
    dirs_source,
    dirs_dest,
    dirs_source_ref,
    dirs_dest_ref,
    dirs_empty,
    dirs_flatten_dir_source_ref,
    dirs_keep_dir_source2dest_ref,
    dryrun,
    skip_existing,
    keep_all,
    flatten,
):
    # run the actual function to test
    n_test_dirs = len(dirs_source)
    # run the actual function to test
    for idx_dir in range(n_test_dirs):
        patch_args = [
            "clifs",
            "mv",
            str(dirs_source[idx_dir]),
            str(dirs_empty[idx_dir] if flatten else dirs_dest[idx_dir]),
            "--recursive",
        ]

        if skip_existing:
            patch_args.append("--skip_existing")
        if keep_all:
            patch_args.append("--keep_all")
        if flatten:
            patch_args.append("--flatten")
        if dryrun:
            patch_args.append("--dryrun")

        with patch("sys.argv", patch_args):
            main()
    print("Copy went through.")

    if not dryrun:
        # check for proper updating and deleting
        if flatten:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_empty[idx_dir],
                    dirs_flatten_dir_source_ref[idx_dir],
                    check_mtime=False,
                )
                assert_files_present(
                    dirs_flatten_dir_source_ref[idx_dir],
                    dirs_empty[idx_dir],
                    check_mtime=False,
                )
        elif keep_all:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_keep_dir_source2dest_ref[idx_dir],
                    dirs_dest[idx_dir],
                    check_mtime=False,
                )
                assert_files_present(
                    dirs_dest[idx_dir],
                    dirs_keep_dir_source2dest_ref[idx_dir],
                    check_mtime=False,
                )
        else:
            for idx_dir in range(n_test_dirs):
                assert_files_present(
                    dirs_source[idx_dir],
                    dirs_dest[idx_dir],
                    check_mtime=not skip_existing,
                )
    else:
        # check for dest dir integrity
        for idx_dir in range(n_test_dirs):
            assert_files_present(dirs_dest[idx_dir], dirs_dest_ref[idx_dir])

        # check for source dir integrity
        for idx_dir in range(n_test_dirs):
            assert_files_present(dirs_source[idx_dir], dirs_source_ref[idx_dir])



"""Command line interface for the file system."""

import importlib.metadata
from pathlib import Path

from clifs.clifs_plugin import ClifsPlugin

try:
    __version__ = importlib.metadata.version("clifs")
except importlib.metadata.PackageNotFoundError:
    with open(Path(__file__).parent / "VERSION", encoding="UTF-8") as version_file:
        __version__ = version_file.read().strip()

__all__ = ["ClifsPlugin"]



# -*- coding: utf-8 -*-


from unittest.mock import patch

from clifs.__main__ import main
from tests.common.utils_testing import parametrize_default_ids, substr_in_dir_names


@parametrize_default_ids("filter_str", ["DELME", "file"])
def test_delete(dirs_dest, filter_str):
    # run the actual function to test

    for dir in dirs_dest:
        with patch("sys.argv", ["clifs", "del", str(dir), "--recursive"]), patch(
            "builtins.input", return_value="yes"
        ):
            main()
        print(f"Delete of {dir.name} went through.")

        assert not substr_in_dir_names(dir, sub_str=filter_str, files_only=True)
